# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yandex/cloud/apploadbalancer/v1/payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from yandex.cloud import validation_pb2 as yandex_dot_cloud_dot_validation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-yandex/cloud/apploadbalancer/v1/payload.proto\x12\x1fyandex.cloud.apploadbalancer.v1\x1a\x1dyandex/cloud/validation.proto\"2\n\x07Payload\x12\x16\n\x04text\x18\x01 \x01(\tB\x06\x8a\xc8\x31\x02>0H\x00\x42\x0f\n\x07payload\x12\x04\xc0\xc1\x31\x01\x42z\n#yandex.cloud.api.apploadbalancer.v1ZSgithub.com/yandex-cloud/go-genproto/yandex/cloud/apploadbalancer/v1;apploadbalancerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'yandex.cloud.apploadbalancer.v1.payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#yandex.cloud.api.apploadbalancer.v1ZSgithub.com/yandex-cloud/go-genproto/yandex/cloud/apploadbalancer/v1;apploadbalancer'
  _PAYLOAD.oneofs_by_name['payload']._options = None
  _PAYLOAD.oneofs_by_name['payload']._serialized_options = b'\300\3011\001'
  _PAYLOAD.fields_by_name['text']._options = None
  _PAYLOAD.fields_by_name['text']._serialized_options = b'\212\3101\002>0'
  _globals['_PAYLOAD']._serialized_start=113
  _globals['_PAYLOAD']._serialized_end=163
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from yandex.cloud.marketplace.metering.v1 import image_product_usage_service_pb2 as yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2


class ImageProductUsageServiceStub(object):
    """A set of methods for managing image product's usage.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Write = channel.unary_unary(
                '/yandex.cloud.marketplace.metering.v1.ImageProductUsageService/Write',
                request_serializer=yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageRequest.SerializeToString,
                response_deserializer=yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageResponse.FromString,
                )


class ImageProductUsageServiceServicer(object):
    """A set of methods for managing image product's usage.
    """

    def Write(self, request, context):
        """Writes image product's usage. Authentication is by user's service account.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ImageProductUsageServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Write': grpc.unary_unary_rpc_method_handler(
                    servicer.Write,
                    request_deserializer=yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageRequest.FromString,
                    response_serializer=yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'yandex.cloud.marketplace.metering.v1.ImageProductUsageService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ImageProductUsageService(object):
    """A set of methods for managing image product's usage.
    """

    @staticmethod
    def Write(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yandex.cloud.marketplace.metering.v1.ImageProductUsageService/Write',
            yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageRequest.SerializeToString,
            yandex_dot_cloud_dot_marketplace_dot_metering_dot_v1_dot_image__product__usage__service__pb2.WriteImageProductUsageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import abc
import collections.abc
import grpc
import grpc.aio
import typing
import yandex.cloud.access.access_pb2
import yandex.cloud.operation.operation_pb2
import yandex.cloud.ydb.v1.backup_pb2
import yandex.cloud.ydb.v1.backup_service_pb2

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore[misc, type-arg]
    ...

class BackupServiceStub:
    """A set of methods for managing backups."""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    Get: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.GetBackupRequest,
        yandex.cloud.ydb.v1.backup_pb2.Backup,
    ]
    """Returns the specified backup."""

    ListPaths: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.ListPathsRequest,
        yandex.cloud.ydb.v1.backup_service_pb2.ListPathsResponse,
    ]

    List: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsRequest,
        yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsResponse,
    ]
    """Retrieves a list of backups."""

    ListAccessBindings: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.ListAccessBindingsRequest,
        yandex.cloud.access.access_pb2.ListAccessBindingsResponse,
    ]

    SetAccessBindings: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.SetAccessBindingsRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]

    UpdateAccessBindings: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.UpdateAccessBindingsRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]

    Delete: grpc.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.DeleteBackupRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]
    """Deletes the specified backup."""

class BackupServiceAsyncStub:
    """A set of methods for managing backups."""

    Get: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.GetBackupRequest,
        yandex.cloud.ydb.v1.backup_pb2.Backup,
    ]
    """Returns the specified backup."""

    ListPaths: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.ListPathsRequest,
        yandex.cloud.ydb.v1.backup_service_pb2.ListPathsResponse,
    ]

    List: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsRequest,
        yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsResponse,
    ]
    """Retrieves a list of backups."""

    ListAccessBindings: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.ListAccessBindingsRequest,
        yandex.cloud.access.access_pb2.ListAccessBindingsResponse,
    ]

    SetAccessBindings: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.SetAccessBindingsRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]

    UpdateAccessBindings: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.access.access_pb2.UpdateAccessBindingsRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]

    Delete: grpc.aio.UnaryUnaryMultiCallable[
        yandex.cloud.ydb.v1.backup_service_pb2.DeleteBackupRequest,
        yandex.cloud.operation.operation_pb2.Operation,
    ]
    """Deletes the specified backup."""

class BackupServiceServicer(metaclass=abc.ABCMeta):
    """A set of methods for managing backups."""

    @abc.abstractmethod
    def Get(
        self,
        request: yandex.cloud.ydb.v1.backup_service_pb2.GetBackupRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.ydb.v1.backup_pb2.Backup, collections.abc.Awaitable[yandex.cloud.ydb.v1.backup_pb2.Backup]]:
        """Returns the specified backup."""

    @abc.abstractmethod
    def ListPaths(
        self,
        request: yandex.cloud.ydb.v1.backup_service_pb2.ListPathsRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.ydb.v1.backup_service_pb2.ListPathsResponse, collections.abc.Awaitable[yandex.cloud.ydb.v1.backup_service_pb2.ListPathsResponse]]: ...

    @abc.abstractmethod
    def List(
        self,
        request: yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsResponse, collections.abc.Awaitable[yandex.cloud.ydb.v1.backup_service_pb2.ListBackupsResponse]]:
        """Retrieves a list of backups."""

    @abc.abstractmethod
    def ListAccessBindings(
        self,
        request: yandex.cloud.access.access_pb2.ListAccessBindingsRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.access.access_pb2.ListAccessBindingsResponse, collections.abc.Awaitable[yandex.cloud.access.access_pb2.ListAccessBindingsResponse]]: ...

    @abc.abstractmethod
    def SetAccessBindings(
        self,
        request: yandex.cloud.access.access_pb2.SetAccessBindingsRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.operation.operation_pb2.Operation, collections.abc.Awaitable[yandex.cloud.operation.operation_pb2.Operation]]: ...

    @abc.abstractmethod
    def UpdateAccessBindings(
        self,
        request: yandex.cloud.access.access_pb2.UpdateAccessBindingsRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.operation.operation_pb2.Operation, collections.abc.Awaitable[yandex.cloud.operation.operation_pb2.Operation]]: ...

    @abc.abstractmethod
    def Delete(
        self,
        request: yandex.cloud.ydb.v1.backup_service_pb2.DeleteBackupRequest,
        context: _ServicerContext,
    ) -> typing.Union[yandex.cloud.operation.operation_pb2.Operation, collections.abc.Awaitable[yandex.cloud.operation.operation_pb2.Operation]]:
        """Deletes the specified backup."""

def add_BackupServiceServicer_to_server(servicer: BackupServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from yandex.cloud.loadtesting.agent.v1 import agent_service_pb2 as yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2


class AgentServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ClaimStatus = channel.unary_unary(
                '/yandex.cloud.loadtesting.agent.v1.AgentService/ClaimStatus',
                request_serializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusRequest.SerializeToString,
                response_deserializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusResponse.FromString,
                )
        self.ReportEventLogs = channel.unary_unary(
                '/yandex.cloud.loadtesting.agent.v1.AgentService/ReportEventLogs',
                request_serializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsRequest.SerializeToString,
                response_deserializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsResponse.FromString,
                )


class AgentServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ClaimStatus(self, request, context):
        """Claims status for the specified agent.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReportEventLogs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AgentServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ClaimStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.ClaimStatus,
                    request_deserializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusRequest.FromString,
                    response_serializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusResponse.SerializeToString,
            ),
            'ReportEventLogs': grpc.unary_unary_rpc_method_handler(
                    servicer.ReportEventLogs,
                    request_deserializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsRequest.FromString,
                    response_serializer=yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'yandex.cloud.loadtesting.agent.v1.AgentService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AgentService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ClaimStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yandex.cloud.loadtesting.agent.v1.AgentService/ClaimStatus',
            yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusRequest.SerializeToString,
            yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ClaimAgentStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReportEventLogs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yandex.cloud.loadtesting.agent.v1.AgentService/ReportEventLogs',
            yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsRequest.SerializeToString,
            yandex_dot_cloud_dot_loadtesting_dot_agent_dot_v1_dot_agent__service__pb2.ReportEventLogsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from yandex.cloud.backup.v1 import provider_service_pb2 as yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2
from yandex.cloud.operation import operation_pb2 as yandex_dot_cloud_dot_operation_dot_operation__pb2


class ProviderServiceStub(object):
    """A set of methods for managing [backup providers](/docs/backup/concepts/#providers).
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Activate = channel.unary_unary(
                '/yandex.cloud.backup.v1.ProviderService/Activate',
                request_serializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ActivateProviderRequest.SerializeToString,
                response_deserializer=yandex_dot_cloud_dot_operation_dot_operation__pb2.Operation.FromString,
                )
        self.ListActivated = channel.unary_unary(
                '/yandex.cloud.backup.v1.ProviderService/ListActivated',
                request_serializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersRequest.SerializeToString,
                response_deserializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersResponse.FromString,
                )


class ProviderServiceServicer(object):
    """A set of methods for managing [backup providers](/docs/backup/concepts/#providers).
    """

    def Activate(self, request, context):
        """Activate provider for specified client.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListActivated(self, request, context):
        """List activated providers for specified client.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProviderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Activate': grpc.unary_unary_rpc_method_handler(
                    servicer.Activate,
                    request_deserializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ActivateProviderRequest.FromString,
                    response_serializer=yandex_dot_cloud_dot_operation_dot_operation__pb2.Operation.SerializeToString,
            ),
            'ListActivated': grpc.unary_unary_rpc_method_handler(
                    servicer.ListActivated,
                    request_deserializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersRequest.FromString,
                    response_serializer=yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'yandex.cloud.backup.v1.ProviderService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ProviderService(object):
    """A set of methods for managing [backup providers](/docs/backup/concepts/#providers).
    """

    @staticmethod
    def Activate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yandex.cloud.backup.v1.ProviderService/Activate',
            yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ActivateProviderRequest.SerializeToString,
            yandex_dot_cloud_dot_operation_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListActivated(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yandex.cloud.backup.v1.ProviderService/ListActivated',
            yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersRequest.SerializeToString,
            yandex_dot_cloud_dot_backup_dot_v1_dot_provider__service__pb2.ListActivatedProvidersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from datetime import datetime
from typing import TYPE_CHECKING, Callable, Optional, Tuple, Union

import grpc
from six.moves.urllib.parse import urlparse

from yandex.cloud.iam.v1.iam_token_service_pb2_grpc import IamTokenServiceStub

if TYPE_CHECKING:
    from yandex.cloud.iam.v1.iam_token_service_pb2 import CreateIamTokenResponse
    from yandexcloud._auth_fabric import (
        IamTokenAuth,
        MetadataAuth,
        ServiceAccountAuth,
        TokenAuth,
    )


TIMEOUT_SECONDS = 20


class Credentials(grpc.AuthMetadataPlugin):
    def __init__(
        self,
        token_requester: Union["MetadataAuth", "TokenAuth", "IamTokenAuth", "ServiceAccountAuth"],
        lazy_channel: Callable[[], "grpc.Channel"],
    ):
        # pylint: disable=super-init-not-called
        self.__token_requester = token_requester
        self._lazy_channel = lazy_channel
        self._channel: Optional[grpc.Channel] = None
        self._cached_iam_token: str = ""
        self._iam_token_timestamp: Optional[datetime] = None

    def __call__(self, context: "grpc.AuthMetadataContext", callback: "grpc.AuthMetadataPluginCallback") -> None:
        try:
            return self._call(context, callback)
        except Exception as exception:  # pylint: disable=broad-except
            callback(tuple(), exception)
        return None

    def _call(self, context: "grpc.AuthMetadataContext", callback: "grpc.AuthMetadataPluginCallback") -> None:
        u = urlparse(context.service_url)
        if u.path in (
            "/yandex.cloud.iam.v1.IamTokenService",
            "/yandex.cloud.endpoint.ApiEndpointService",
        ):
            callback(tuple(), None)
            return

        if self._channel is None:
            self._channel = self._lazy_channel()

        if not self._fresh():
            get_token = getattr(self.__token_requester, "get_token", None)
            if callable(get_token):
                self._cached_iam_token = get_token()
                self._iam_token_timestamp = datetime.now()
                callback(self._metadata(), None)
                return

            get_token_request = getattr(self.__token_requester, "get_token_request", None)
            if callable(get_token_request):
                token_future = IamTokenServiceStub(self._channel).Create.future(get_token_request())
                token_future.add_done_callback(self.create_done_callback(callback))
                return

        callback(self._metadata(), None)

    def create_done_callback(self, callback: "grpc.AuthMetadataPluginCallback") -> Callable[["grpc.Future"], None]:
        def done_callback(future: "grpc.Future") -> None:
            try:
                resp = future.result()
            except Exception as exception:  # pylint: disable=broad-except
                callback(tuple(), exception)
            else:
                self._save_token(resp)
                callback(self._metadata(), None)

        return done_callback

    def _metadata(self) -> Tuple[Tuple[str, str]]:
        metadata = (("authorization", f"Bearer {self._cached_iam_token}"),)
        return metadata

    def _save_token(self, resp: "CreateIamTokenResponse") -> None:
        self._cached_iam_token = resp.iam_token
        self._iam_token_timestamp = datetime.now()

    def _fresh(self) -> bool:
        if self._cached_iam_token == "":
            return False
        if self._iam_token_timestamp is None:
            return False
        diff = datetime.now() - self._iam_token_timestamp
        return diff.total_seconds() < TIMEOUT_SECONDS
